function [Yq,ent,rmse] = dctquant(Yr,step,N)

% function [Yq,ent,rmse] = dctquant(Yr,step,N)
% Quantise and calculate the entropy of each of the N x N bands of DCT coefs.
% Assumes that Yr has been regrouped into separate sub-images.
% step defines the entropy step-size.
% If step is an N x N matrix then each element defines the stepsize for
% the corresponding DCT band.

if size(step) ~= [N N], step = step(1) * ones(N,N); end

Yq = zeros(size(Yr));
if nargout > 1, ent = zeros(N,N); end
rmse = zeros(N,N);

[m,n] = size(Yr); m = m/N; n = n/N;
tr = 1:m;
for i=1:N,
  tc = 1:n;
  if nargout > 1,
    for k=1:N,
      Yi = round(Yr(tr,tc) / step(i,k));
      Yq(tr,tc) = Yi * step(i,k);
      rmse(i,k) = rms(rms(Yq(tr,tc) - Yr(tr,tc)));
      [n0,x0] = hist2d(Yi);
      ent(i,k) = sum(entrop(n0));
      fprintf(1,'.');
      tc = tc + n;
    end
  else
    for k=1:N,
      Yi = round(Yr(tr,tc) / step(i,k));
      Yq(tr,tc) = Yi * step(i,k);
      rmse(i,k) = rms(rms(Yq(tr,tc) - Yr(tr,tc)));
      fprintf(1,'.');
      tc = tc + n;
    end
  end
  tr = tr + m;
  fprintf(1,'\n');
end

return;
