% DEMJPEG3.M - J-PEG simulation with Qstep = qscale * JPEG luminance table
% where qscale is input by the user.

clear all
close all
global printon
printon = input('Write print files (0=no, 1=yes):');
qscale = input('JPEG Q-table scale factor:');

% JPEG quantisation table for luminance.
qstep = qscale*jpegqtab;
% dcbits = 6;
dcbits = round(7 - log(qscale)/log(2));
dcmax = 2^(dcbits-1) - 1;
dcmin = -dcmax - 1;
qstep, dcbits

% Default Huffman tables:
 hufflum;
% huffchr;

% 1-D DCT matrix.
N = 8;
T = dctmat(N);

load lenna
X = X - 120;

disp('Doing DCT'); X=colxfm(colxfm(X,T)',T)'; 

% Use 2 iterations to get huffman tables based on actual data.
for iter=1:2,

disp('Generating huffval data')

% Generate zig-zag scan of AC coefs.
if N==8,
  scan=[9 2 3 10 17 25 18 11 4 5 12 19 26 33 41 34 27 20 13 6];
  scan=[scan 7 14 21 28 35 42 49 57 50 43 36 29 22 15 8];
  scan=[scan 16 23 30 37 44 51 58 59 52 45 38 31 24];
  scan=[scan 32 39 46 53 60 61 54 47 40 48 55 62 63 56 64];
end

disp('Generating AC encoder huffman table')
global ehuf
ehuf = ehufgen(bits, huffval);

disp('Generating DC encoder huffman table')
global ehufdc
ehufdc = ehufgen(bitsdc, huffvdc);

% Quantise each block, generate run/ampl values and code them into
% codestr(:,1:2).  Also generate a histogram of code symbols.
disp('Coding rows')
sy = size(X);
t = 1:N;
global huffhist
huffhist = zeros(16*16,1);
global huffhidc
huffhidc = zeros(16,1);

vlc = [];
sqerr = zeros(N,N);
for r=0:N:(sy(1)-N),
  fprintf(1,' %d',r)
  vlc1 = [];
  dcpred = 0;
  for c=0:N:(sy(2)-N),
    yq = round(X(r+t,c+t) ./ qstep);
%    dccoef = min(dcmax,max(dcmin,yq(1)));
    dccoefs(r/N+1,c/N+1) = yq(1) - dcpred;
    radc = runampl(yq(1) - dcpred);
    dcpred = yq(1);
    ra1 = runampl(yq(scan));
    vlc1 = [vlc1; huffendc(radc); huffenc(ra1)];
    err = (X(r+t,c+t) - yq .* qstep);
    sqerr = sqerr + err .* err;
  end
  vlc = [vlc; vlc1];
end
disp('Coding done')

rmse = sqrt(sqerr*(N*N/prod(sy)));
rmsem = rms(rmse(:))
nbits = sum(vlc(:,2));
bitrate = nbits/prod(sy);
fprintf(1,'No. of bits for coded image = %d\n', nbits);
fprintf(1,'Bit rate = %.4f bit/pel\n', bitrate);

% Plot histogram of use of huffman codes.
ncodes = sum(huffhist);
setfig(iter+1)
bar(([1:length(huffhist)]-2)/16, huffhist/ncodes)
title(sprintf('Histogram of %d JPEG Huffman codes; Bit rate = %.4f b/pel.',...
 ncodes, bitrate));
xlabel('Run length');
ylabel('Code probability');
text(-1/8,huffhist(1)/ncodes,'EOB','vert','mid','horiz','right');
text(1/16,huffhist(2)/ncodes,'size 1','vert','mid','horiz','left');
text(2/16,huffhist(3)/ncodes,'size 2','vert','mid','horiz','left');
text(3/16,huffhist(4)/ncodes,'size 3','vert','mid','horiz','left');
text(4/16,huffhist(5)/ncodes,'size 4','vert','mid','horiz','left');
text(17/16,huffhist(18)/ncodes,'size 1','vert','mid','horiz','left');
axis([-0.8 6.5 0 0.3501]);
grid on
drawnow

if iter == 1,
  [bits, huffval] = genhuff(huffhist); 
  [bitsdc, huffvdc] = genhuff2(huffhidc); 
  bitsdc, huffvdc'
  disp('Using custom tables for DC and AC luminance');
else
  if printon==1,
    print -deps \1ngk\tex\i8coding\i8jpeg2.eps
  end
end

end % End of iteration loop.

return;

