echo on
% DEMO1.M
% I8 Coding Course - Lecture 1, Colour Demonstrations:
% Effects of quantisation -- contouring, show Claire quantised to
% 6, 4, 2 bits in Y and in CrCb.  Also show effects of reducing
% the sampling rate by 4:1 and 16:1 for Y and CrCb.

clear all
% Generate 192x192 yuv image of claire in y,u,v.
[y,u,v] = x2yuv(b24read('cl192'));
% save cl192y y
% save cl192uv u v

% Calculate histograms of Y and UV images.
bins = 7:16:247;
figure(1); hist(y(:),bins); pause
title('Histogram of luminance (Y) image of Claire')
figure(2); hist(u(:),bins); pause
title('Histogram of blue chrominance (U) image of Claire')
figure(3); hist(v(:),bins); pause
title('Histogram of red chrominance (V) image of Claire')

% Generate pure Y and pure U, V, and UV images.
q = zeros(size(y)) + 128;
b24write(yuv2x(y,q,q),'cly00');
b24write(yuv2x(q,u,q),'cl0u0');
b24write(yuv2x(q,q,v),'cl00v');
b24write(yuv2x(q,u,v),'cl0uv');

% Quantise luminance to 4 and 2 bits.
q = 16*floor(y/16) + 7;
b24write(yuv2x(q,u,v),'clq4y');

q = 64*floor(y/64) + 31;
b24write(yuv2x(q,u,v),'clq2y');

% Quantise chrominance to 4 and 2 bits.
q = 16*floor(u/16) + 7;
qv = 16*floor(v/16) + 7;
b24write(yuv2x(y,q,qv),'clq4uv');

q = 64*floor(u/64) + 31;
qv = 64*floor(v/64) + 31;
b24write(yuv2x(y,q,qv),'clq2uv');

% Quantise luminance to 4 bits after adding random noise.
q = 16*floor(y/16 + rand(size(y)) - 0.5) + 7;
b24write(yuv2x(q,u,v),'clq4ny');

% Quantise luminance to 4 bits after adding dither.
q = 16*floor(y/16 + kron(ones(size(y)/2), [-3 -1;1 3]/8)) + 7;
b24write(yuv2x(q,u,v),'clq4dy');

% Decimate y,u,v by 4 and then 4 again.
h = [1 7 7 1]/16;
h2 = 2*h;
y4 = rowdec(rowdec(y,h).',h).';
u4 = rowdec(rowdec(u,h).',h).';
v4 = rowdec(rowdec(v,h).',h).';
q = rowint(rowint(y4.',h2).',h2);
b24write(yuv2x(q,u,v),'cld4y');
q = rowint(rowint(u4.',h2).',h2);
qv = rowint(rowint(v4.',h2).',h2);
b24write(yuv2x(y,q,qv),'cld4uv');

y16 = rowdec(rowdec(y4,h).',h).';
u16 = rowdec(rowdec(u4,h).',h).';
v16 = rowdec(rowdec(v4,h).',h).';
y4 = rowint(rowint(y16.',h2).',h2);
u4 = rowint(rowint(u16.',h2).',h2);
v4 = rowint(rowint(v16.',h2).',h2);
q = rowint(rowint(y4.',h2).',h2);
b24write(yuv2x(q,u,v),'cld16y');
q = rowint(rowint(u4.',h2).',h2);
qv = rowint(rowint(v4.',h2).',h2);
b24write(yuv2x(y,q,qv),'cld16uv');

% Assemble the results back into large 400x600 images.

% Split image into Y, U, V, and U+V components.
clear all
grey = 256*257 + 1;
y = zeros(460,600) + grey*240;
t = 1:192;
y(t+4,t+4) = b24read('cl192');
y(t+4,t+204) = b24read('cly00');
y(t+234,t+4) = b24read('cl0u0');
y(t+234,t+204) = b24read('cl00v');
y(t+234,t+404) = b24read('cl0uv');
b24write(y,'demo1a');

% Quantise Y and U+V to 16 and 4 levels.
clear all
grey = 256*257 + 1;
y = zeros(460,600) + grey*240;
t = 1:192;
y(t+4,t+4) = b24read('cly00');
y(t+4,t+204) = b24read('clq4y');
y(t+4,t+404) = b24read('clq2y');
y(t+234,t+4) = b24read('cl0uv');
y(t+234,t+204) = b24read('clq4uv');
y(t+234,t+404) = b24read('clq2uv');
b24write(y,'demo1b');

% Decimate Y and U+V by 4 and 16.
clear all
grey = 256*257 + 1;
y = zeros(460,600) + grey*240;
t = 1:192;
y(t+4,t+4) = b24read('cly00');
y(t+4,t+204) = b24read('cld4y');
y(t+4,t+404) = b24read('cld16y');
y(t+234,t+4) = b24read('cl0uv');
y(t+234,t+204) = b24read('cld4uv');
y(t+234,t+404) = b24read('cld16uv');
b24write(y,'demo1c');

% Activity Masking -- quote result and show the effect of adding noise
% on reducing visibility of contouring.
% 
% Noise and Dither demo.
clear all
grey = 256*257 + 1;
y = zeros(460,400) + grey*240;
t = 1:192;
y(t+4,t+4) = b24read('cl192');
y(t+4,t+204) = b24read('clq4y');
y(t+234,t+4) = b24read('clq4ny');
y(t+234,t+204) = b24read('clq4dy');
b24write(y,'demo1d');

return


