% DEMWAV2.M
% Perform analysis of lenna using the following wavelets:
%   Haar wavelet,
%   Le Gall analysis and reconstruction wavelets,
%   Dabechies 4-tap wavelets,
%   Tay/Kingsbury near-balanced 5,7-tap wavelets,
%   Tay/Kingsbury near-balanced 13,19-tap wavelets.

clear all
close all

global printon
printon = input('Write print files (0=no, 1=norm, 2=hi Qstep, 3=RLC):');

% Use run-lengths for lecture note results.
run = input('Calculate entropies based on run-lengths (1=yes, 0=no):');

% 50,2,2 are the default values for the figures in lecture notes 4.
qstep = input('Quantiser step size (50?) = ');
qlev = input('No. of quantiser scales (2?) = ');
qinc = input('Quantiser step increment (2?) = ');
if qinc == 1,
  qstr = ['Qstep=' int2str(qstep)];
else
  qstr = sprintf('Qstep=%d,%.2f',qstep,qinc);
  qstep = dwtqtab(qstep,qlev,qinc)
end

pause on

% Start with the Haar wavelet.

h0haar = [1;1]/sqrt(2);
h1haar = [1;-1]/sqrt(2);

% Now do the Le Gall wavelet.
h0legall = [1 2 1]'/sqrt(8);
h1legall = [-1 -2 6 -2 -1]'/sqrt(32);

% This is the other Le Gall wavelet - less smooth.
g1legall = [-1 2 -1]'/sqrt(8);
g0legall = [-1 2 6 2 -1]'/sqrt(32);

% Define the Daubechies 4-tap wavelet.
alpha = sqrt(3) - 2;
h0daub = conv([1 2 1]',[1 alpha]') / (2 * sqrt(1 + alpha^2));
h1daub = h0daub(4:-1:1) .* [-1 1 -1 1]';

% Now do the 5,7-tap and 13,19-tap Transformation of variables filters.
c = -3.5;
a = 2*c + 2/(2 + c);
b = -(2 + c);
Ht = conv([1 1]',-[1 b a]');
Ft = conv([1 1]',-[1 c]');

% Define the transformation function Z = M(z) (even coefs must be 0).
% for the 5,7-tap filters.
M = [1 0 1]'/2;

% Calculate the transformed low and high synthesis filters, lo(z) and hi(z).
h0 = xfm(Ft,M);
h1 = xfm(Ht,-M);

% Normalise to unit energy.
h0_57 = h0/sqrt(sum(h0 .* h0));
h1_57 = h1/sqrt(sum(h1 .* h1));

% Now swap the filters to demonstrate symmetry between analysis
% and reconstruction filters.
% Calculate the transformed low and high synthesis filters, lo(z) and hi(z).
h0 = xfm(Ht,M);
h1 = xfm(Ft,-M);

% Normalise to unit energy.
h0_75 = h0/sqrt(sum(h0 .* h0));
h1_75 = h1/sqrt(sum(h1 .* h1));

% Define the transformation function Z = M(z) (even coefs must be 0).
% for the 13,19-tap filters.
ma=3/16;
M = [-ma 0 1+ma 0 1+ma 0 -ma]'/2;

% Calculate the transformed low and high synthesis filters, lo(z) and hi(z).
h0 = xfm(Ft,M);
h1 = xfm(Ht,-M);

% Normalise to unit energy.
h0_13 = h0/sqrt(sum(h0 .* h0));
h1_19 = h1/sqrt(sum(h1 .* h1));

% Now swap the filters to demonstrate symmetry between analysis
% and reconstruction filters.
% Calculate the transformed low and high synthesis filters, lo(z) and hi(z).
h0 = xfm(Ht,M);
h1 = xfm(Ft,-M);

% Normalise to unit energy.
h0_19 = h0/sqrt(sum(h0 .* h0));
h1_13 = h1/sqrt(sum(h1 .* h1));

% Perform 4 level DWT and calculate entropies ee and eh for each filter
% type.
h0 = h0haar;
h1 = h1haar;
g0 = h0haar;
g1 = -h1haar;
xfmstr = 'Haar DWT';
dwt4ent;
save enthaar2 ee eh X qstep h0 h1 g0 g1 xfmstr

h0 = h0legall;
h1 = h1legall;
g0 = g0legall;
g1 = g1legall;
xfmstr = 'LeGall DWT';
dwt4ent;
save entlegal ee eh X qstep h0 h1 g0 g1 xfmstr

h0 = g0legall;
h1 = g1legall;
g0 = h0legall;
g1 = h1legall;
xfmstr = 'Inv-LeGall DWT';
dwt4ent;
save entinleg ee eh X qstep h0 h1 g0 g1 xfmstr

h0 = h0_57;
h1 = h1_57;
g0 = h0_75;
g1 = h1_75;
xfmstr = 'Near-bal 5,7-tap DWT';
dwt4ent;
save ent_57 ee eh X qstep h0 h1 g0 g1 xfmstr

h0 = h0_13;
h1 = h1_19;
g0 = h0_19;
g1 = h1_13;
xfmstr = 'Near-bal 13,19-tap DWT';
dwt4ent;
save ent_1319 ee eh X qstep h0 h1 g0 g1 xfmstr

return;


h0 = h0daub;
h1 = h1daub;
g0 = h0daub(4:-1:1);
g1 = h1daub(4:-1:1);
xfmstr = 'Daubechies 4-tap DWT';
Y = dwt(X,h0,h1);
Y = idwt(Y,g0,g1);
figure(1)
draw(Y-X);

return;



h0 = h0daub;
h1 = h1daub;
g0 = h0daub(4:-1:1);
g1 = h1daub(4:-1:1);
xfmstr = 'Daubechies 4-tap DWT';
dwt4ent;
save entdaub ee eh rmserr

