% DEMWAV3.M
% Perform 2-D wavelet displays of the following wavelets:
%   Haar wavelet,
%   Le Gall analysis and reconstruction wavelets,
%   Dabechies 4-tap wavelets,
%   Tay/Kingsbury near-balanced 5,7-tap wavelets,
%   Tay/Kingsbury near-balanced 13,19-tap wavelets.

clear all
% close all

global printon
% printon = input('Write print files (0=no, 1=yes, 2=GIF):');
printon=0;

pause on

% Start with the Haar wavelet.

h0haar = [1;1]/sqrt(2);
h1haar = [1;-1]/sqrt(2);

% Now do the Le Gall wavelet.
g0legall = [1 2 1]'/sqrt(8);
g1legall = [-1 -2 6 -2 -1]'/sqrt(32);

% This is the other Le Gall wavelet - less smooth.
h1legall = [-1 2 -1]'/sqrt(8);
h0legall = [-1 2 6 2 -1]'/sqrt(32);

% Define the Daubechies 4-tap wavelet.
alpha = sqrt(3) - 2;
h0daub = conv([1 2 1]',[1 alpha]') / (2 * sqrt(1 + alpha^2));
h1daub = h0daub(4:-1:1) .* [-1 1 -1 1]';

% Now do the 5,7-tap and 13,19-tap Transformation of variables filters.
c = -3.5;
a = 2*c + 2/(2 + c);
b = -(2 + c);
Ht = conv([1 1]',-[1 b a]');
Ft = conv([1 1]',-[1 c]');

% Define the transformation function Z = M(z) (even coefs must be 0).
% for the 5,7-tap filters.
M = [1 0 1]'/2;

% Calculate the transformed low and high synthesis filters, lo(z) and hi(z).
h0 = xfm(Ft,M);
h1 = xfm(Ht,-M);

% Normalise to unit energy.
h0_57 = h0/sqrt(sum(h0 .* h0));
h1_57 = h1/sqrt(sum(h1 .* h1));

% Now swap the filters to demonstrate symmetry between analysis
% and reconstruction filters.
% Calculate the transformed low and high synthesis filters, lo(z) and hi(z).
h0 = xfm(Ht,M);
h1 = xfm(Ft,-M);

% Normalise to unit energy.
h0_75 = h0/sqrt(sum(h0 .* h0));
h1_75 = h1/sqrt(sum(h1 .* h1));

% Define the transformation function Z = M(z) (even coefs must be 0).
% for the 13,19-tap filters.
ma=3/16;
M = [-ma 0 1+ma 0 1+ma 0 -ma]'/2;

% Calculate the transformed low and high synthesis filters, lo(z) and hi(z).
h0 = xfm(Ft,M);
h1 = xfm(Ht,-M);

% Normalise to unit energy.
h0_13 = h0/sqrt(sum(h0 .* h0));
h1_19 = h1/sqrt(sum(h1 .* h1));

% Now swap the filters to demonstrate symmetry between analysis
% and reconstruction filters.
% Calculate the transformed low and high synthesis filters, lo(z) and hi(z).
h0 = xfm(Ht,M);
h1 = xfm(Ft,-M);

% Normalise to unit energy.
h0_19 = h0/sqrt(sum(h0 .* h0));
h1_13 = h1/sqrt(sum(h1 .* h1));

h = extend(wav4(g0legall,g1legall),61);
t = (size(h,1)+1)/2 + [-30:30]';
hhlegall = [h(t,5); h(t,4)]*[h(t,5); h(t,4)]';

h = wav4(h0_57,h1_57);
t = (size(h,1)+1)/2 + [-30:30]';
hh_57 = [h(t,5); h(t,4)]*[h(t,5); h(t,4)]';

h = wav4(h0_13,h1_19);
t = (size(h,1)+1)/2 + [-30:30]';
hh_1319 = [h(t,5); h(t,4)]*[h(t,5); h(t,4)]';

% setfig(1)
figure;setfig(gcf)
vline = 0.1 * ones(size(hh_1319,1),1);
draw([hhlegall vline hh_57 vline hh_1319])
pos=[150 20 854 416];
set(gcf,'position',pos);
title('Legall 3,5-tap, and near balaced 5,7-tap and 13,19-tap 2-D wavelets at level 4','FontSize',16);
if printon==2,
  drawfile([hhlegall vline hh_57],'\1ngk\tex\i8coding\i8wav31a.gif');
  drawfile([hh_57 vline hh_1319],'\1ngk\tex\i8coding\i8wav31b.gif');
elseif printon==1,
  print -deps \1ngk\tex\i8coding\i8wav31.eps
else
  pause
end

% Now look at freq responses.

% setfig(2)
figure;setfig(gcf)
ax = [-0.5 0.5 -0.5 0.5 0 2.5];
t = [-16:15]/32; 
subplot(2,2,1)
mesh(t,t,fftshift(abs(fft2(g0legall*g0legall',32,32))))
axis(ax)
titlestr='Frequency responses of 2-D LeGall 3,5-tap wavelets';
text(1.2,1.07,titlestr,'Units','norm','horiz','center','vert','bottom')
xlabel('Lo-Lo band','FontSize',16)
drawnow
subplot(2,2,2)
mesh(t,t,fftshift(abs(fft2(g0legall*g1legall',32,32))))
axis(ax)
xlabel('Hi-Lo band','FontSize',16)
subplot(2,2,3)
mesh(t,t,fftshift(abs(fft2(g1legall*g0legall',32,32))))
axis(ax)
xlabel('Lo-Hi band','FontSize',16)
subplot(2,2,4)
mesh(t,t,fftshift(abs(fft2(g1legall*g1legall',32,32))))
axis(ax)
xlabel('Hi-Hi band','FontSize',16)
if printon==1,
  print -deps \1ngk\tex\i8coding\i8wav32.eps
else
  pause
end

% setfig(3)
figure;setfig(gcf)
ax = [-0.5 0.5 -0.5 0.5 0 2.5];
t = [-16:15]/32; 
subplot(2,2,1)
mesh(t,t,fftshift(abs(fft2(h0_13*h0_13',32,32))))
axis(ax)
titlestr='Frequency responses of 2-D near-balanced 13,19-tap wavelets';
text(1.2,1.07,titlestr,'Units','norm','horiz','center','vert','bottom')
xlabel('Lo-Lo band','FontSize',16)
drawnow
subplot(2,2,2)
mesh(t,t,fftshift(abs(fft2(h0_13*h1_19',32,32))))
axis(ax)
xlabel('Hi-Lo band','FontSize',16)
subplot(2,2,3)
mesh(t,t,fftshift(abs(fft2(h1_19*h0_13',32,32))))
axis(ax)
xlabel('Lo-Hi band','FontSize',16)
subplot(2,2,4)
mesh(t,t,fftshift(abs(fft2(h1_19*h1_19',32,32))))
axis(ax)
xlabel('Hi-Hi band','FontSize',16)
if printon==1,
  print -deps \1ngk\tex\i8coding\i8wav33.eps
else
  pause
end

% setfig(4)
figure;setfig(gcf)
ax = [-0.5 0.5 -0.5 0.5];
conts = ([1:10]-0.5)/4;
t = [-16:15]/32; 
subplot(2,2,1)
contour(t,t,fftshift(abs(fft2(g0legall*g0legall',32,32))),conts)
axis(ax)
axis('square')
grid
titlestr='Contours of frequency responses of 2-D LeGall 3,5-tap wavelets';
text(1.4,1.07,titlestr,'Units','norm','horiz','center','vert','bottom')
xlabel('Lo-Lo band','FontSize',16)
subplot(2,2,2)
contour(t,t,fftshift(abs(fft2(g0legall*g1legall',32,32))),conts)
axis(ax)
axis('square')
grid
xlabel('Hi-Lo band','FontSize',16)
subplot(2,2,3)
contour(t,t,fftshift(abs(fft2(g1legall*g0legall',32,32))),conts)
axis(ax)
axis('square')
grid
xlabel('Lo-Hi band','FontSize',16)
subplot(2,2,4)
contour(t,t,fftshift(abs(fft2(g1legall*g1legall',32,32))),conts)
axis(ax)
axis('square')
grid
xlabel('Hi-Hi band','FontSize',16)
if printon==1,
  print -deps \1ngk\tex\i8coding\i8wav34.eps
else
  pause
end

% setfig(5)
figure;setfig(gcf)
ax = [-0.5 0.5 -0.5 0.5];
conts = ([1:10]-0.5)/4;
t = [-16:15]/32; 
subplot(2,2,1)
contour(t,t,fftshift(abs(fft2(h0_13*h0_13',32,32))),conts)
axis(ax)
axis('square')
grid
titlestr='Contours of frequency responses of 2-D near-balanced 13,19-tap wavelets';
text(1.4,1.07,titlestr,'Units','norm','horiz','center','vert','bottom')
xlabel('Lo-Lo band','FontSize',16)
subplot(2,2,2)
contour(t,t,fftshift(abs(fft2(h0_13*h1_19',32,32))),conts)
axis(ax)
axis('square')
grid
xlabel('Hi-Lo band','FontSize',16)
subplot(2,2,3)
contour(t,t,fftshift(abs(fft2(h1_19*h0_13',32,32))),conts)
axis(ax)
axis('square')
grid
xlabel('Lo-Hi band','FontSize',16)
subplot(2,2,4)
contour(t,t,fftshift(abs(fft2(h1_19*h1_19',32,32))),conts)
axis(ax)
axis('square')
grid
xlabel('Hi-Hi band','FontSize',16)
if printon==1,
  print -deps \1ngk\tex\i8coding\i8wav35.eps
end

return;



