% DWT4ENT.M
% M-file to perform 4 levels of DWT and calulate entropies
% and to reconstruct the images and measure the MSE.

% Load Lenna image.
load lenna
X = X - 120; % Remove DC bias.
si = size(X);
sq = size(qstep,1);

% Perform DWT level 1.
X = dwt(X,h0,h1);

figure(1);
[e1, ehuff1] = ent4plot(X,qstep(1,:),...
 [xfmstr ', Level 1 energies, and entropies for ' qstr],...
 [-15 15*(1+run) 0 1],run);
drawnow;

% figure(2);draw(max(min(X,200),-200));pause

% Repeat for 2 to 4 levels of transform.
% Perform DWT level 2.
sx = round(size(X)/2);
X(1:sx(1),1:sx(2)) = dwt(X(1:sx(1),1:sx(2)),h0,h1);

if sq < 2, qs = qstep(sq,1); else qs = qstep(2,:); end
figure(1);
[e2, ehuff2] = ent4plot(X(1:sx(1),1:sx(2)),qs, ...
 [xfmstr ', Level 2 energies, and entropies for ' qstr], ...
 [-30 30*(1+run) 0 0.8], run);
drawnow;

% figure(2);draw(max(min(X(1:sx(1),1:sx(2)),400),-400));pause

% Perform DWT level 3.
sx = round(sx/2);
X(1:sx(1),1:sx(2)) = dwt(X(1:sx(1),1:sx(2)),h0,h1);

if sq < 3, qs = qstep(sq,1); else qs = qstep(3,:); end
figure(1);
[e3, ehuff3] = ent4plot(X(1:sx(1),1:sx(2)),qs, ...
 [xfmstr ', Level 3 energies, and entropies for ' qstr], ...
 [-60 60 0 0.6], run);
drawnow;

% figure(2);draw(max(min(X(1:sx(1),1:sx(2)),800),-800));pause

% Perform DWT level 4.
sx = round(sx/2);
X(1:sx(1),1:sx(2)) = dwt(X(1:sx(1),1:sx(2)),h0,h1);

if sq < 4, qs = qstep(sq,1); else qs = qstep(4,:); end
figure(1);
[e4, ehuff4] = ent4plot(X(1:sx(1),1:sx(2)),qs, ...
 [xfmstr ', Level 4 energies, and entropies for ' qstr], ...
 [-120 120 0 0.5], run);
drawnow;

% figure(2);draw(max(min(X(1:sx(1),1:sx(2)),1600),-1600));pause

% Draw a bar plot showing the entropy components from each subimage
% as a function of number of levels.
ee=zeros(13,5);
ee(1,1) = 0;
ee(4:-1:1,2:5) = e1 * (ones(1,4)/4);
ee(7:-1:4,3:5) = e2 * (ones(1,3)/16);
ee(10:-1:7,4:5) = e3 * (ones(1,2)/64);
ee(13:-1:10,5) = e4 / 256;

eh=zeros(13,5);
eh(1,1) = 0;
eh(4:-1:1,2:5) = ehuff1 * (ones(1,4)/4);
eh(7:-1:4,3:5) = ehuff2 * (ones(1,3)/16);
eh(10:-1:7,4:5) = ehuff3 * (ones(1,2)/64);
eh(13:-1:10,5) = ehuff4 / 256;

xfmstr,ee,sum(ee),eh,sum(eh)

% Compute the inverse transform of the image X quantised with Qstep.
X = dwtquant(X,qstep);

disp('Performing inverse DWT')
X(1:sx(1),1:sx(2)) = idwt(X(1:sx(1),1:sx(2)),g0,g1);
sx = sx*2;
X(1:sx(1),1:sx(2)) = idwt(X(1:sx(1),1:sx(2)),g0,g1);
sx = sx*2;
X(1:sx(1),1:sx(2)) = idwt(X(1:sx(1),1:sx(2)),g0,g1);
sx = sx*2;
X = min(max(round(idwt(X,g0,g1)+120),1),255);
figure(2)
draw(X);
pause

return;

