function Y=dwtquant(X,qstep)

% function Y=dwtquant(X,qstep)
% Quantise X with steps of size qstep and return the reconstructed
% image as Y.
% If qstep is scalar the whole of X is quantised uniformly.
% Else qstep should be a  M x 4 matrix, with qstep(m,i)
% being used to quantise subband i at level m in an M-level DWT.
% qstep(m,1) for m=1:(M-1) are not used.

% Nick Kingsbury, Cambridge University, Feb 96.

% Uniform quantisation if qstep is a scalar.
if length(qstep) == 1,
  Y = qstep*round(X/qstep);
  return;
end

M = size(qstep);
if M(2) ~= 4, error('qstep must be of width 4, if not a scalar.'); end
M = M(1);
sx = size(X);

% Loop for each level.
for m=1:M,
  sx = round(sx/2);
  t1 = 1:sx(1);
  t2 = 1:sx(2);

% Exclude Lo-Lo band from quantisation except at final level.
  if m == M, 
    Y(t1,t2) = qstep(m,1)*round(X(t1,t2)/qstep(m,1));
  end

  Y(t1,t2+sx(2)) = qstep(m,2)*round(X(t1,t2+sx(2))/qstep(m,2));
  Y(t1+sx(1),t2) = qstep(m,3)*round(X(t1+sx(1),t2)/qstep(m,3));
  Y(t1+sx(1),t2+sx(2)) = qstep(m,4)*round(X(t1+sx(1),t2+sx(2))/qstep(m,4));
end

return;
