function ehuf = ehufgen(bits, huffval)

% function ehuf = ehufgen(bits, huffval)
% Generate the encoder huffman table ehuf(:,2) for JPEG encoding using
% the JPEG code definitions: bits and huffval.
% ehuf(:,1) contains the codewords, and
% ehuf(:,2) contains their lengths in bits.

% Generate huffman size table (JPEG fig C1, JPEG Book p406):
nb=length(bits);
k=1;
j=1;
ncodes=sum(bits);
huffsize=zeros(ncodes,1);
for i=1:nb,
  while j<=bits(i), huffsize(k)=i; k=k+1; j=j+1; end	
  j=1;
end

% Generate huffman code table (JPEG fig C2, JPEG Book p407):
huffcode=zeros(ncodes,1);
code=0;
si=huffsize(1);
for k=1:ncodes,
  while huffsize(k)>si,
    code=code*2;
    si=si+1;
  end
  huffcode(k)=code;
  code=code+1;
end

% Reorder the code tables according to the data in huffval
% to yield the encoder look-up tables.
% disp('Reordering ehuf')
ehuf=zeros(256,2);
ehuf(huffval+1,:)=[huffcode huffsize];

return;


