function [no,xo] = hist2d(y,ni,xi)

% function [no,xo] = hist2d(y,ni,xi)
% Plot a histogram of the elements of matrix y which must have
% been rounded to integer values.
% If no output arguments, the graph is plotted, else the
% results are returned as column vectors [no,xo] so that bar(xo,no) 
% plots the histogram.
% If ni,xi are given the new histogram counts are added as a extra
% column to those in ni,xi to give no,xo.

% Modified from hist.m by Nick Kingsbury, Cambridge University
% Dec 95.
% 

xmin = min(min(y));
xmax = max(max(y));

x = [xmin:xmax].';
nn = zeros(size(x));

for i=1:length(x),
  nn(i) = sum(sum(y == x(i)));
end

% Add ni,xi if given.
if nargin > 2,
  xomin = min([x; xi]);
  xomax = max([x; xi]);
  xo = [xomin:xomax].';
  no = zeros(size(xo));
  t = [1:length(x)]+x(1)-xo(1);
  no(t) = nn;
  t = [1:length(xi)]+xi(1)-xo(1);
  no(t) = ni + no(t);
else
  if nargout == 0
    bar(x,nn);
  else
    xo = x;
    no = nn;
  end
end

return;

