% J-PEG simulation.

% DCT.
 N = 8;
 C8=dctmat(N);
 disp('Doing DCT'); Y=colxfm(colxfm(X,C8)',C8)'; 

% Quantise to integers.
  disp('Quantising'); Yq=quant1(Y,16,16);

disp('Generating huffval data')
sy=size(Yq);
% syN=sy/N;

% Generate zig-zag scan of AC coefs.
if N==8,
  scan=[9 2 3 10 17 25 18 11 4 5 12 19 26 33 41 34 27 20 13 6];
  scan=[scan 7 14 21 28 35 42 49 57 50 43 36 29 22 15 8];
  scan=[scan 16 23 30 37 44 51 58 59 52 45 38 31 24];
  scan=[scan 32 39 46 53 60 61 54 47 40 48 55 62 63 56 64];
end

bits=hex2vec('0 2 1 3 3 2 4 3 5 5 4 4 0 0 1 7D');

huffval=hex2vec('1 2 3 0 4 11 5 12 21 31 41 6 13 51 61 7')';
huffval=[huffval; hex2vec('22 71 14 32 81 91 a1 08 23 42 b1 c1 15 52 d1 f0')'];
huffval=[huffval; hex2vec('24 33 62 72 82 09 0a 16 17 18 19 1a 25 26 27 28')'];
huffval=[huffval; hex2vec('29 2a 34 35 36 37 38 39 3a 43 44 45 46 47 48 49')'];
huffval=[huffval; hex2vec('4a 53 54 55 56 57 58 59 5a 63 64 65 66 67 68 69')'];
huffval=[huffval; hex2vec('6a 73 74 75 76 77 78 79 7a 83 84 85 86 87 88 89')'];
huffval=[huffval; hex2vec('8a 92 93 94 95 96 97 98 99 9a a2 a3 a4 a5 a6 a7')'];
huffval=[huffval; hex2vec('a8 a9 aa b2 b3 b4 b5 b6 b7 b8 b9 ba c2 c3 c4 c5')'];
huffval=[huffval; hex2vec('c6 c7 c8 c9 ca d2 d3 d4 d5 d6 d7 d8 d9 da e1 e2')'];
huffval=[huffval; hex2vec('e3 e4 e5 e6 e7 e8 e9 ea f1 f2 f3 f4 f5 f6 f7 f8')'];
huffval=[huffval; hex2vec('f9 fa')'];

disp('Generating huffsize and huffcode')

% Generate huffman size table (JPEG fig C1, p78):
nb=length(bits);
k=1;
j=1;
ncodes=sum(bits);
huffsize=zeros(ncodes,1);
for i=1:nb,
  while j<=bits(i), huffsize(k)=i; k=k+1; j=j+1; end	
  j=1;
end

% Generate huffman code table (JPEG fig C2, p79):
huffcode=zeros(ncodes,1);
code=0;
si=huffsize(1);
for k=1:ncodes,
  while huffsize(k)>si,
    code=code*2;
    si=si+1;
  end
  huffcode(k)=code;
  code=code+1;
end

% Reorder the code tables according to the data in huffval
% to yield the encoder look-up tables.
disp('Reordering ehuf')
global ehuf
ehuf=zeros(256,2);
ehuf(huffval+1,:)=[huffcode huffsize];

% Generate run/ampl values and code them into codestr(:,1:2).
% Also generate a histogram of code symbols.
disp('Coding rows')
t = 1:N;
global huffhist
huffhist = zeros(16*16,1);
vlc = [];
for r=0:N:(sy(1)-N),
  fprintf(1,' %d',r)
  vlc1 = [];
  for c=0:N:(sy(2)-N),
    yq = Yq(r+t,c+t);
    dccoef = min(255,max(0,yq(1)));
    ra1 = runampl(yq(scan));
    vlc1 = [vlc1; [dccoef 8]; huffenc(ra1)];
  end
  vlc = [vlc; vlc1];
end
disp('Coding done')

fprintf(1,'No. of bits for coded image = %d\n', sum(vlc(:,2)))

% Plot histogram of use of huffman codes.
bar(0:length(huffhist)-1, huffhist)
title('Histogram of huffman code usage')