% Laplace.m
% Laplacian PDF entropy plot.

if exist('printon') ~= 1, printon = 0; end

% Plot p(x) = \fracd{1}{2 x_0}~e^{-|x|/x_0}
xmax = 6;
x = [-xmax:0.1:xmax]';
p = exp(-abs(x))/2;

setfig(1);

% Plot axes.
plot ([0 0],[0.6 -0.2],':r',[-xmax xmax],[0 0],':r')
axis('normal')
axis([-xmax xmax -0.2 0.6])
axis('off')
set(gca,'position',[0.05 0.1 0.9 0.8]);
hold on

% Plot quantiser thresholds.
plot([1;1]*[-(xmax-1):2:(xmax-1)],[-0.1;0.3]*ones(1,xmax),'--c')

% Plot Laplacian pdf.
plot (x, p, '-w');
hold off
latext(-0.1,0.5,'1/(2x_0)','r');
text(-5,-0.11,'-5Q/2','horiz','c','vert','t');
text(-3,-0.11,'-3Q/2','horiz','c','vert','t');
text(-1,-0.11,'-Q/2','horiz','c','vert','t');
text(1,-0.11,'Q/2','horiz','c','vert','t');
text(3,-0.11,'3Q/2','horiz','c','vert','t');
text(5,-0.11,'5Q/2','horiz','c','vert','t');
text(0.5,0.4,'Laplacian PDF, p(x)','horiz','l');
text(-4,0.31,'Quantiser thresholds','horiz','c','vert','bo');


xdb = [-30:0.5:30]';
x = 10.^(xdb/20);
q=ones(size(x))./x;
H = entlapl(q);
Hline = log(2*exp(1)*x)/log(2);
% The following is a much closer approximation:
% Hline = log((2*exp(1)*x).^2.5 + 1)/(2.5*log(2));
pp = 2.5; Hline = log((2*exp(1)*x).^pp + 1)/(pp*log(2));
setfig(2);
plot(xdb,[H Hline]); 
axis([-30 30 -4 8])
grid on
latext(0,-5,'x_0 / Q  (dB)');
latext(-15,0.8,'H');
latext(-15,-0.6,'H_a');
latext(-35,3,'Entropy\\(bit/pel)');

if printon,
  figure(1); print -deps \1ngk\tex\i8coding\i8fLap1.eps
  figure(2); print -deps \1ngk\tex\i8coding\i8fLap2.eps
end



return
 

