% MOVI.M
% Display colour movies for I8 Video demo.
% Define fr (vector of frames), rpt (no of repeats).
% Display colour movie images using IMAGE in its full-colour mode.
% In particular this selects various versions of the suzie sequence
% (original, coded at 64kb/s, and coded at 32kb/s), according to
% the current figure selected (gcf).
% Use:  figure;movi   eight times to set up the 8 figure windows 
% correctly.  Control it using fr and rpt.

fig = gcf;
if fig.Number==1, load suzie50; fr=1:40; settitle('Original Suzie Sequence');
elseif fig.Number==2, load suz64; fr=1:40; settitle('64kb/s Coded');
elseif fig.Number==3, load suz32; fr=1:40; settitle('32kb/s Coded');
elseif fig.Number==4, load suzie50; fr=1:20; settitle('Frame 20');
elseif fig.Number==5, load suzie50; fr=1:21; settitle('Frame 21');
elseif fig.Number==6, load suzr0m; settitle('Residual without MC');
elseif fig.Number==7, load suzrmc; settitle('Residual with MC');
elseif fig.Number==8, load suzmc; settitle('MC Prediction Frame');
end

sseq = size(seq);
nframes = fix(sseq(3)/3);
if ~exist('fr'), fr = 1:nframes; end
if ~exist('rpt'), rpt = 1; end
if nframes > 1, fr1 = min(fr, nframes);
else fr1 = 1;
end

figure(gcf)
rgb = 1:3;
imh=image(seq(:,:,rgb));axis image;axis off
if ~exist('dtime'), dtime = 0.1; end % Frame period (sec).
% tic
t1 = clock;
t2 = dtime;
for rep = 1:rpt,
for frame = fr1, 
   set(imh,'CData',seq(:,:,(frame-1)*3+rgb),'EraseMode','none');
   drawnow
   while etime(clock,t1) < t2, dummy = 1; end
   t2 = t2 + dtime;
end
fr1 = fr1(length(fr1):-1:1);  % Reverse play at each repeat.
end
% toc
return
