function [ep, ehuff] = runent(a)

% function [ep, ehuff] = runent(a)
% Calculate the entropy components ep of a quantised vector a(:)
% assuming that run-length coding is used for the runs of zeros
% and normal entropy coding is used for the non-zero values.
% The total entropy is sum(ep).
% ehuff is the huffman code equivalent ep. 
% Plot the entropy histogram (normal histogram with
% counts weighted by -log2(p)/N so that the bar heights indicate the
% contribution to the entropy of each bar).

N = prod(size(a));

% Generate indices to non-zero elements of a.
b = find(a(:));
if isempty(b),
  ep = 0;
  ehuff = 0;
  return
end

% Calculate histogram of non-zero elements.
[na,xa] = hist2d(a(b));

% Convert b to a vector of runlengths and eliminate zeros.
b = diff([0;b])-1;
b = b(find(b > 0));

% Any runs > 128, should be replaced by repeated use of 128 and a remainder.
big = find(b > 128);
for ib = big',
  q = floor(b(ib)/128);
  re = rem(b(ib),128);
  b(ib) = 128;
  if q > 1, b = [b; 128*ones(q-1,1)]; end
  if re > 0, b = [b; re]; end
end

% Calculate histogram of run lengths.
[nb,xb] = hist2d(b);

% Calculate the total number of non-zero elements and runs.
nnz = sum(na);
nrun = sum(nb);

% Calculate entropy of non-zero elements and run lengths.
na = [na;nb];
xa = [xa; [1:length(nb)]'+max(xa)];
ep = entrop(na,xa);

% Calculate the Huffman entropies.
[bits, huffval, ehuff] = genhuff2(na); % huffent = huffman code entropy.

% Scale the entropies down by (nnz + nrun)/N so that the entropies
% are on a per pel basis.
ep = ep * ((nnz + nrun)/N);
ehuff = ehuff * ((nnz + nrun)/N);

% N, nnz, nrun

return
