% SHIFTVAR.M
% Investigate the shift variance of subband energy for a variety of
% common wavelets.
%
% Nick Kingsbury, Cambridge University, June 1996.

% Generate a series of n steps with 1-bit shifts, extended by ex samples.
n = 16;
% ex = 64;
% x = cumsum([zeros(ex,n); diag(ones(n,1)); zeros(ex,n)]);

% Now do the Antonini wavelets.
% This polynomial makes even terms of P(z) zero when other factors 
% are (1+z)^8.
r2 = roots([5 -40 131 -208 131 -40 5]);

% Antonini 7,9-tap wavelets (FBI standard).
% 4 roots at z=-1 and the 4 complex roots from r2.
h0ant = real(poly([-1; -1; -1; -1; r2(2:5)])');
% 4 roots at z=-1 and the 2 real roots from r2.
g0ant = real(poly([-1; -1; -1; -1; r2([1 6])])');

% Antonini 6,10-tap wavelets.
% 5 roots at z=-1 and the 4 complex roots from r2.
% h0ant = real(poly([-1; -1; -1; -1; -1; r2(2:5)])');
% 3 roots at z=-1 and the 2 real roots from r2.
% g0ant = real(poly([-1; -1; -1; r2([1 6])])');

h0ant = sqrt(2)*h0ant/sum(h0ant);  % sqrt(2) DC gain.
g0ant = sqrt(2)*g0ant/sum(g0ant);  % sqrt(2) DC gain.

% Generate hi-band filters as quadrature mirrors of lo-band filters.
h1ant = g0ant .* cumprod(-ones(size(g0ant)));
g1ant = -h0ant .* cumprod(-ones(size(h0ant)));

h = wav4(g0ant,g1ant);

% Step function matrix, extended to 160 samples.
hs=cumsum([h;zeros(160-size(h,1),5)]);
setfig(1)
plot([-64:95]/16,hs(:,4))
grid

% Energy matrix.
es = hs .* hs;

% Derive the energy fluctuations at each level.
t1 = 0:2:127;
t2 = 1:4:127;
t3 = 3:8:127;
t4 = 7:16:127;
for i=1:16,
  efl(i,:) = [sum(es(t1+i,1)) sum(es(t2+i,2)) sum(es(t3+i,3)) sum(es(t4+i,4))]; 
end

setfig(2)
cplot([efl; efl],-1);
grid

