% yuq2movi.m
% Read a .YUQ file and save it as a movie for use by MOVI.M.
% Convert QCIF YUV (.YUQ) files to RGB format and save them as .MAT
% files, suitable for use with MOVI.M.
% Define name (filename) and nframes (no of frames to convert).

% name = 'suz32';
fid = fopen(['\1ngk\visual\' name '.yuq'],'r');
% nframes = 40;

width = 16 * 11;
height = 16 * 9;

% Convert  y,u,v into an rgb image,
% using definitions from the JPEG Book, pp 18-19.
% Actually Cb,Cr are used instead of U,V.
% Y = 0.3R + 0.6G + 0.1B;  U = (B - Y)/2 + 127.5;
% V = (B - Y)/1.6 + 127.5; therefore
% R = Y + (V-127.5)*1.6; B = Y + (U-127.5)*2;
% G = (Y - 0.3*R - 0.1*B)/0.6 = Y - (V-127.5)*0.8 - (U-127.5)/3;
% The results are rounded to the nearest integers and are limited
% to the range 0 to 255.

Kr = 0.3;
Kb = 0.1;

Ku = 2;
Kv = 1.6;
Ko = 0.5 * 255;
Ku2 = Kb * Ku / (1 - Kr - Kb);
Kv2 = Kr * Kv / (1 - Kr - Kb);

im =  uint8(zeros(height,width,3));
seq = [];
% M = moviein(nframes,gcf);

for frame = 1:nframes,
 fr3 = frame * 3;  
 for row = 1:2:height,
   y = fread(fid,[width 2],'uchar').';
   uv = fread(fid,width,'uchar').';
   u = kron(uv(2:2:width),ones(2,2));
   v = kron(uv(1:2:width),ones(2,2));
   r = row + [0:1];
   im(r,:,1) = uint8(max(min(round(y + (v - Ko)*Kv),255),0));
   im(r,:,2) = uint8(max(min(round(y - (v - Ko)*Kv2 - (u - Ko)*Ku2),255),0));
   im(r,:,3) = uint8(max(min(round(y + (u - Ko)*Ku),255),0));
 end
 
 seq = cat(3,seq,im);
 frame
 
 figure(1);
 image(im);axis image;axis off
 drawnow
 
end

fclose(fid);

eval(['save ' name ' seq']); 

return

